/*  
 * Copyright (c) 2011 Ant Kutschera
 * 
 * This file is part of Ant Kutschera's blog, 
 * http://blog.maxant.co.uk
 * 
 * This is free software: you can redistribute
 * it and/or modify it under the terms of the
 * Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 * 
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the Lesser GNU General Public License for
 * more details. 
 * 
 * You should have received a copy of the
 * Lesser GNU General Public License along with this software.
 * If not, see http://www.gnu.org/licenses/.
 */
package ch.maxant.comms.client;

import ch.maxant.comms.client.internal.DefaultRequestMarshaller;
import ch.maxant.comms.client.internal.DefaultResponseUnmarshaller;
import ch.maxant.comms.client.internal.DefaultServiceNameResolver;
import ch.maxant.comms.common.Constants;
import ch.maxant.comms.common.SerializationType;

/**
 * TODO XSD for the config xml
 *
 */
public class ClientConfiguration {

	private String host;
	private int port;
	private String path;
	private String username;
	private String password;
	private boolean ignoreServerCertificate;
	private SerializationType serializationType = SerializationType.JAVA_SERIALIZATION;
	private boolean useZipping;
	private String marshallerClass = DefaultRequestMarshaller.class.getName();
	private String unmarshallerClass = DefaultResponseUnmarshaller.class.getName();
	private boolean secure;
	private String serviceNameResolverClass = DefaultServiceNameResolver.class.getName();
	private String charEncoding = Constants.DEFAULT_CHAR_ENCODING;
	
	public boolean isIgnoreServerCertificate() {
		return ignoreServerCertificate;
	}
	
	/** 
	 * if true, then server certificates do not need to be in the 
	 * truststore in order for us to call the server, we just ignore them
	 */
	public void setIgnoreServerCertificate(boolean ignoreServerCertificate) {
		this.ignoreServerCertificate = ignoreServerCertificate;
	}
	
	/** @return default {@link SerializationType#JAVA_SERIALIZATION} */
	public SerializationType getSerializationType() {
		return serializationType;
	}
	public void setSerializationType(SerializationType serializationType) {
		this.serializationType = serializationType;
	}
	public boolean isUseZipping() {
		return useZipping;
	}
	public void setUseZipping(boolean useZipping) {
		this.useZipping = useZipping;
	}

	/** @return default {@link DefaultRequestMarshaller} */
	public String getMarshallerClass() {
		return marshallerClass;
	}
	public void setMarshallerClass(String marshallerClass) {
		this.marshallerClass = marshallerClass;
	}

	/** @return default {@link DefaultResponseUnmarshaller} */
	public String getUnmarshallerClass() {
		return unmarshallerClass;
	}
	public void setUnmarshallerClass(String unmarshallerClass) {
		this.unmarshallerClass = unmarshallerClass;
	}
	public boolean isSecure() {
		return secure;
	}
	public void setSecure(boolean secure) {
		this.secure = secure;
	}

	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public int getPort() {
		return port;
	}

	public void setPort(int port) {
		this.port = port;
	}

	public String getPath() {
		return path;
	}

	public void setPath(String path) {
		this.path = path;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getServiceNameResolverClass() {
		return serviceNameResolverClass;
	}
	
	public void setServiceNameResolverClass(String serviceNameResolverClass) {
		this.serviceNameResolverClass = serviceNameResolverClass;
	}

	public String getCharEncoding() {
		return charEncoding;
	}

	public void setCharEncoding(String charEncoding) {
		this.charEncoding = charEncoding;
	}

}
