/*  
 * Copyright (c) 2011 Ant Kutschera
 * 
 * This file is part of Ant Kutschera's blog, 
 * http://blog.maxant.co.uk
 * 
 * This is free software: you can redistribute
 * it and/or modify it under the terms of the
 * Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 * 
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the Lesser GNU General Public License for
 * more details. 
 * 
 * You should have received a copy of the
 * Lesser GNU General Public License along with this software.
 * If not, see http://www.gnu.org/licenses/.
 */
package ch.maxant.comms.client.internal;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.lang.reflect.Method;

import org.apache.commons.codec.binary.Base64;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import ch.maxant.comms.client.ClientConfiguration;
import ch.maxant.comms.client.RequestMarshaller;
import ch.maxant.comms.client.ServiceNameResolver;
import ch.maxant.comms.common.SerializationType;

public class DefaultRequestMarshaller implements RequestMarshaller {

	private static final String NEW_LINE = "\r\n";
	
	@Override
	public String marshal(ClientConfiguration config, Class<?> serviceInterface, Method method,
			Object[] args, ServiceNameResolver nameResolver) throws Exception {
		
		StringBuilder out = new StringBuilder();
		
		out.append(config.getSerializationType()).append(NEW_LINE);
		
		out.append(nameResolver.getName(serviceInterface)).append(NEW_LINE);
		out.append(method.getName()).append(NEW_LINE);
		out.append(method.getParameterTypes() == null ? 0 : method.getParameterTypes().length).append(NEW_LINE);
		if(method.getParameterTypes() != null){
			for(Class<?> c : method.getParameterTypes()){
				out.append(c.getName()).append(NEW_LINE);
			}
		}
		if(args != null){
			for(Object arg : args){
				String marshalledArg = marshalArg(config.getSerializationType(), arg);
				out.append(marshalledArg).append(NEW_LINE);
			}
		}
		
		return out.toString();
	}

	private String marshalArg(SerializationType type, Object arg) throws IOException {

		byte[] raw = null;
		if(SerializationType.JAVA_SERIALIZATION.equals(type)){
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			ObjectOutputStream oos = new ObjectOutputStream(baos);
			oos.writeObject(arg);
			oos.flush();
			oos.close();
			raw = baos.toByteArray();
		}else if(SerializationType.EMF_XMI.equals(type)){
			
			ResourceSet resourceSet = new ResourceSetImpl();
			
			// Register the appropriate resource factory to handle all file extensions.
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put
			(Resource.Factory.Registry.DEFAULT_EXTENSION, 
					new XMIResourceFactoryImpl());
			
//			resourceSet.getPackageRegistry().put
//			(AddressbookPackage.eNS_URI, 
//					AddressbookPackage.eINSTANCE);
			
			//cant save to this: URI uri = URI.createPlatformPluginURI("/ch.sbb.e4.addressbook.data/model/Addressbook_test001.xmi", true);
//			URI uri = URI.createFileURI("D:/training3/e4_workshop_course/ws/ch.sbb.e4.addressbook.data/model/Addressbook_test001.xmi");
//			resource = resourceSet.getResource(uri, true);

			Resource resource = resourceSet.createResource(URI.createURI("asdf"));
			
			resource.getContents().add((EObject)arg);
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			resource.save(baos, null);
			raw = baos.toByteArray();
			
		}else{
			throw new UnsupportedOperationException(String.valueOf(type));
		}

		return new String(Base64.encodeBase64(raw));
	
	}
}
