/*  
 * Copyright (c) 2011 Ant Kutschera
 * 
 * This file is part of Ant Kutschera's blog, 
 * http://blog.maxant.co.uk
 * 
 * This is free software: you can redistribute
 * it and/or modify it under the terms of the
 * Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 * 
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the Lesser GNU General Public License for
 * more details. 
 * 
 * You should have received a copy of the
 * Lesser GNU General Public License along with this software.
 * If not, see http://www.gnu.org/licenses/.
 */
package ch.maxant.comms.client.internal;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;

import org.apache.commons.codec.binary.Base64;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import ch.maxant.comms.client.ClientConfiguration;
import ch.maxant.comms.client.ResponseUnmarshaller;
import ch.maxant.comms.common.SerializationType;

public class DefaultResponseUnmarshaller implements ResponseUnmarshaller {

	@Override
	public Object unmarshal(ClientConfiguration config, String response) throws Throwable {
		
		if(response == null || response.length() == 0){
			return null;
		}
		
		//take the basee64 encoded string and decode it and deserialise it
		byte[] raw = response.getBytes(config.getCharEncoding());
		raw = Base64.decodeBase64(raw);
		Object ret = null;
		if(SerializationType.JAVA_SERIALIZATION.equals(config.getSerializationType())){
			ByteArrayInputStream bais = new ByteArrayInputStream(raw);
			ObjectInputStream ois = new ObjectInputStream(bais);
			ret = ois.readObject();
			ois.close();
		}else if(SerializationType.EMF_XMI.equals(config.getSerializationType())){
			
			ResourceSet resourceSet = new ResourceSetImpl();
			
			// Register the appropriate resource factory to handle all file extensions.
			resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put
			(Resource.Factory.Registry.DEFAULT_EXTENSION, 
					new XMIResourceFactoryImpl());

			//TODO seems to be ok for marshalling, without this bit. what does this bit actually do?
//			resourceSet.getPackageRegistry().put
//			(AddressbookPackage.eNS_URI, 
//					AddressbookPackage.eINSTANCE);
			
			//cant save to this: URI uri = URI.createPlatformPluginURI("/ch.sbb.e4.addressbook.data/model/Addressbook_test001.xmi", true);
//			URI uri = URI.createFileURI("D:/training3/e4_workshop_course/ws/ch.sbb.e4.addressbook.data/model/Addressbook_test001.xmi");
//			resource = resourceSet.getResource(uri, true);

			Resource resource = resourceSet.createResource(URI.createURI("asdf"));
			
			ByteArrayInputStream bais = new ByteArrayInputStream(raw);
			resource.load(bais, null);

			ret = resource.getContents().get(0);
			
		}else{
			throw new UnsupportedOperationException(String.valueOf(config.getSerializationType()));
		}
		
		if(ret instanceof Throwable){
			throw (Throwable)ret;
		}
		return ret;
	}
}
