/*  
 * Copyright (c) 2011 Ant Kutschera
 * 
 * This file is part of Ant Kutschera's blog, 
 * http://blog.maxant.co.uk
 * 
 * This is free software: you can redistribute
 * it and/or modify it under the terms of the
 * Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 * 
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the Lesser GNU General Public License for
 * more details. 
 * 
 * You should have received a copy of the
 * Lesser GNU General Public License along with this software.
 * If not, see http://www.gnu.org/licenses/.
 */
package ch.maxant.comms.server.internal;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.codec.binary.Base64;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import ch.maxant.comms.common.SerializationType;
import ch.maxant.comms.server.Request;
import ch.maxant.comms.server.RequestUnmarshaller;

public class DefaultRequestUnmarshaller implements RequestUnmarshaller{

	@Override
	public Request unmarshal(HttpServletRequest request) throws IOException, ClassNotFoundException {
		
		Request req = new Request();

		BufferedReader reader = request.getReader();
		
		//////////////////////
		// read serialisation type
		//////////////////////
		String type = reader.readLine();
		req.setSerializationType(SerializationType.valueOf(type));
		
		//////////////////////
		// read service name
		//////////////////////
		String serviceName = reader.readLine();
		req.setServiceName(serviceName);
		
		//////////////////////
		// read method name
		//////////////////////
		req.setMethodName(reader.readLine());
		
		//////////////////////
		// read arguments
		//////////////////////
		Integer numArgs = Integer.parseInt(reader.readLine());
		for(int i = 0; i < numArgs; i++){
			Class<?> clazz = Class.forName(reader.readLine());
			req.getParameterTypes().add(clazz);
		}
		
		//////////////////////
		// read arguments
		//////////////////////
		List<String> args = new ArrayList<String>();
		String curr = null;
		while((curr = reader.readLine()) != null){
			args.add(curr);
		}

		//////////////////////
		// unmarshal arguments
		//////////////////////
		List<Object> arguments = parseArgs(req.getSerializationType(), args, request);
		req.getArguments().addAll(arguments);
		
		return req;
	}
	
	private List<Object> parseArgs(SerializationType type, List<String> args, HttpServletRequest request) throws IOException, ClassNotFoundException  {
		List<Object> ret = new ArrayList<Object>();
		for(String arg : args){
			byte[] raw = Base64.decodeBase64(arg.getBytes(getCharEnc(request)));

			//TODO maybe it would be better to check for instanceof on EObject, coz we wont necessarily be given objects who are ALL EObjects...?
			
			if(SerializationType.JAVA_SERIALIZATION.equals(type)){
				ByteArrayInputStream bais = new ByteArrayInputStream(raw);
				ObjectInputStream ois = new ObjectInputStream(bais);
				Object o = ois.readObject();
				ois.close();
				ret.add(o);
			}else if(SerializationType.EMF_XMI.equals(type)){
				ResourceSet resourceSet = new ResourceSetImpl();
				
				// Register the appropriate resource factory to handle all file extensions.
				resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put
				(Resource.Factory.Registry.DEFAULT_EXTENSION, 
						new XMIResourceFactoryImpl());
				
//				resourceSet.getPackageRegistry().put
//				(AddressbookPackage.eNS_URI, 
//						AddressbookPackage.eINSTANCE);
				
				Resource resource = resourceSet.createResource(URI.createURI("asdf"));
				
				ByteArrayInputStream bais = new ByteArrayInputStream(raw);
				resource.load(bais, null);

				ret.add(resource.getContents().get(0));
			}else{
				throw new UnsupportedOperationException(String.valueOf(type));
			}
		}
		
		return ret;
	}

	private String getCharEnc(HttpServletRequest request) {
		String enc = request.getCharacterEncoding();
		if(enc == null){
			enc = "UTF-8";
		}
		return enc;
	}
}
