/*  
 * Copyright (c) 2011 Ant Kutschera
 * 
 * This file is part of Ant Kutschera's blog, 
 * http://blog.maxant.co.uk
 * 
 * This is free software: you can redistribute
 * it and/or modify it under the terms of the
 * Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 * 
 * It is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the Lesser GNU General Public License for
 * more details. 
 * 
 * You should have received a copy of the
 * Lesser GNU General Public License along with this software.
 * If not, see http://www.gnu.org/licenses/.
 */
package ch.maxant.comms.server.internal;

import java.util.Hashtable;
import java.util.Map;

import ch.maxant.comms.server.ServiceLocator;

public class DefaultServiceLocator implements ServiceLocator {

	//static coz we want to reuse the cache and the server keeps instantiating the service locator
	private static Map<String, Object> cache = new Hashtable<String, Object>();
	
	/** 
	 * {@inheritDoc} default impl simply instantiates an instance of the class name serviceName + "Impl" 
	 * @throws ClassNotFoundException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException
	 */
	@Override
	public Object lookup(String serviceName) throws ClassNotFoundException, InstantiationException, IllegalAccessException {

		Object svc = cache.get(serviceName);
		if(svc == null){
			int i = serviceName.lastIndexOf(".");
			String pckage = serviceName.substring(0, i);
			String clss = serviceName.substring(i+1);
			String fullName = pckage + ".impl." + clss + "Impl";
			
			Class<?> clazz = Class.forName(fullName);
			svc = clazz.newInstance();
			
			cache.put(serviceName, svc);
		}
		
		return svc;
	}
}
